import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.io.*;

public class ImageCanvas extends Canvas
                         implements ClipboardOwner, KeyListener
{
    Clipboard       clipboard;
    Image           img;
    MediaTracker    tracker = new MediaTracker(this);

    public ImageCanvas ()
    {
        addKeyListener(this);
        clipboard = getToolkit().getSystemClipboard();
    }

    public void setImage (Image img)
    {
        tracker.addImage(img, 0);
        try
        {
            tracker.waitForAll();
        }
        catch (Exception e)
        {
            System.out.println("Error loading image: " + e.getMessage());
            return;
        }

        if (tracker.isErrorAny())
        {
            System.out.println("Error loading image.");
            return;
        }

        this.img = img;
        repaint();
    }

    public void paint (Graphics g)
    {
        if (img != null)
        {
            Dimension d = this.getSize();
            g.setColor(Color.white);
            g.fillRect(0,0, d.width, d.height);     // blank background
            g.drawImage(img, 0, 0, img.getWidth(this), img.getHeight(this), this);
        }
    }

    public void keyPressed (KeyEvent evt)
    {
        if (evt.isControlDown())
        {
            if (evt.getKeyCode() == KeyEvent.VK_A)  // copy command
            {
                try
                {
                    clipboard.setContents(new TransferableImage(img), this);
                }
                catch (Exception e)
                {
                    System.out.println("Error copying data. Error: " + e.getMessage());
                }
                evt.consume();
            }
            else if (evt.getKeyCode() == KeyEvent.VK_B) // paste command
            {
                Transferable clipboardData = clipboard.getContents(this);
                        DataFlavor[] flavors = clipboardData.getTransferDataFlavors();
                        for (int j = 0; j < flavors.length; j++)
                        {
                            System.out.println(flavors[j].getMimeType());
                            System.out.println(flavors[j].getHumanPresentableName());
                        }
                if (clipboardData != null)
                {
                    try
                    {
                        this.setImage((Image)clipboardData.getTransferData(new DataFlavor("application/x-java-image;class=java.awt.Image")));
                    }
                    catch (Exception e)
                    {
                        System.out.println("Error pasting data. Error: " + e.getMessage());
                    }
                }
                evt.consume();
            }
        }
    }

    public void keyReleased (KeyEvent evt)  {   }
    public void keyTyped (KeyEvent evt)     {   }

    public void lostOwnership(Clipboard clipboard, Transferable contents)
    {   }

    public static void main (String args[])
    {
        Frame f = new Frame("Image Copy");
        f.setLayout(new GridLayout(0,1));

        ImageCanvas canvas1 = new ImageCanvas();
        ImageCanvas canvas2 = new ImageCanvas();

        canvas1.setImage(Toolkit.getDefaultToolkit().getImage("images/cloud.gif"));
        canvas2.setImage(Toolkit.getDefaultToolkit().getImage("images/sun.gif"));

        f.add(canvas1);
        f.add(canvas2);

        f.setSize(300,300);
        f.show();
    }

}

class TransferableImage implements Transferable
{
    DataFlavor  imageFlavor;
    Image       img;
    XData       data = new XData();

    public TransferableImage (Image img) throws Exception
    {
        this.img = img;
        data.height = 10;
        data.width = 10;
        data.pixels = new int[10];
        imageFlavor = new DataFlavor("text/plain; class=java.io.InputStream; charset=unicode");
        //imageFlavor = new DataFlavor("application/x-java-image; class=java.awt.Image");
        //imageFlavor = new DataFlavor(DataFlavor.javaRemoteObjectMimeType + "; class=XData");
    }

    public Object getTransferData(DataFlavor flavor)
    {
        if (imageFlavor.equals(flavor))
        {
            return (new StringReader("Hello"));
            //return (data);
            //return (img);
            //byte[] b = new byte[100];
            //for (int i = 0; i < b.length; i++)
            //    b[i] = 0;

            //return (new ByteArrayInputStream(b));
        }
        else
            return (null);
    }

    public DataFlavor[] getTransferDataFlavors()
    {
        DataFlavor[] flavors = new DataFlavor[1];
        flavors[0] = imageFlavor;
        return (flavors);
    }

    public boolean isDataFlavorSupported(DataFlavor flavor)
    {
        return (imageFlavor.equals(flavor));
    }
}

class XData implements Serializable
{
    int height;
    int width;
    int[] pixels;
}
