import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.awt.image.*;

public class ImageCanvas extends Canvas
                         implements ClipboardOwner, KeyListener
{
    Clipboard           clipboard;
    Image               img;
    MediaTracker        tracker = new MediaTracker(this);

    public ImageCanvas ()
    {
        addKeyListener(this);
        clipboard = new NativeExtendedClipboard();
    }

    public void setImage (Image img)
    {
        tracker.addImage(img, 0);
        try
        {
            tracker.waitForAll();
        }
        catch (Exception e)
        {
            System.out.println("Error loading image: " + e.getMessage());
            return;
        }

        if (tracker.isErrorAny())
        {
            System.out.println("Error loading image.");
            return;
        }

        this.img = img;
        repaint();
    }

    public void paint (Graphics g)
    {
        if (img != null)
        {
            Dimension d = this.getSize();
            g.setColor(Color.white);
            g.fillRect(0,0, d.width, d.height);     // blank background
            g.drawImage(img, 0, 0, img.getWidth(this), img.getHeight(this), this);
        }
    }

    public void keyPressed (KeyEvent evt)
    {
        if (evt.isControlDown())
        {
            if (evt.getKeyCode() == KeyEvent.VK_A)  // copy command
            {
                try
                {
                    int width = img.getWidth(this);
                    int height = img.getHeight(this);
                    int[] pixels = new int[width * height];
                    PixelGrabber pg =
                        new PixelGrabber(img, 0, 0, width, height, pixels, 0, width);
                    pg.grabPixels(0);

                    clipboard.setContents(
                        new TransferableImage(width, height, pixels), this);
                }
                catch (Exception e)
                {
                    System.out.println("Error copying data. Error: " + e.getMessage());
                }
                evt.consume();
            }
            else if (evt.getKeyCode() == KeyEvent.VK_B) // paste command
            {
                Transferable clipboardData = clipboard.getContents(this);
                if (clipboardData != null)
                {
                    try
                    {
                        DataFlavor flavor = new DataFlavor("image/raw");
                        TransferableImage ti =
                            (TransferableImage) clipboardData.getTransferData(flavor);
                        MemoryImageSource mis =
                            new MemoryImageSource(ti.width, ti.height, ti.pixels, 0, ti.width);
                        this.setImage(Toolkit.getDefaultToolkit().createImage(mis));
                    }
                    catch (Exception e)
                    {
                        System.out.println("Error pasting: " + e.getMessage());
                    }
                }
                evt.consume();
            }
        }
    }

    public void keyReleased (KeyEvent evt)  {   }
    public void keyTyped (KeyEvent evt)     {   }

    public void lostOwnership(Clipboard clipboard, Transferable contents)
    {   }

    public static void main (String args[])
    {
        Frame f = new Frame("Image Copy");
        f.setLayout(new GridLayout(0,1));

        ImageCanvas canvas1 = new ImageCanvas();
        ImageCanvas canvas2 = new ImageCanvas();

        canvas1.setImage(Toolkit.getDefaultToolkit().getImage("images/cloud.gif"));
        canvas2.setImage(Toolkit.getDefaultToolkit().getImage("images/sun.gif"));

        f.add(canvas1);
        f.add(canvas2);

        f.setSize(300,300);
        f.show();
    }
}
