import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class SwingNotifyUser implements NotifyUser
{
    SwingNotifyFrame f;
    NotifyCallback   callback;

    public void addNotify (String iconFileName, String message)
    {
        f = new SwingNotifyFrame("Notify", message, iconFileName);
        f.installNotifyCallback(callback);
    }

    public void removeNotify ()
    {
        if (f != null)
            f.dispose();
    }

    public void installNotifyCallback (NotifyCallback callback)
    {
        this.callback = callback;
        if (f != null)
            f.installNotifyCallback(callback);
    }
}

class SwingNotifyFrame extends JFrame implements ActionListener
{
    JButton viewMailButton = new JButton("View Mail");
    JButton cancelButton = new JButton("Cancel");
    NotifyCallback  callback;

    SwingNotifyFrame (String frameTitle, String message, String iconFileName)
    {
        super(frameTitle);

        JPanel pCenter = new JPanel();
        JPanel pButtons = new JPanel();

        pCenter.setLayout(new FlowLayout());
        pButtons.setLayout(new FlowLayout());

        pCenter.add(new JLabel(new ImageIcon(iconFileName)));
        pCenter.add(new JLabel(message));

        pButtons.add(viewMailButton);
        pButtons.add(cancelButton);

        viewMailButton.addActionListener(this);
        cancelButton.addActionListener(this);

        this.getContentPane().add("Center", pCenter);
        this.getContentPane().add("South", pButtons);

        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Dimension screenSize = toolkit.getScreenSize();

        setSize(300,125);
        setLocation((screenSize.width - 300) / 2, (screenSize.height - 125) / 2);
        setVisible(true);
    }

    public void actionPerformed (ActionEvent evt)
    {
        if (evt.getSource() == viewMailButton)
        {
            if (callback != null)
                callback.restoreApplication();
        }
        else
            this.dispose();
    }

    void installNotifyCallback (NotifyCallback callback)
    {
        this.callback = callback;
    }
}